@@ -1,6 +1,5 @@
 use super::*;
-use crate::{config::Config, repo::*, workspace::Workspace};
-use anyhow::*;
+use crate::{config::Config, workspace::Workspace};
 use std::{io, path};
 
 pub fn exec(origin: &str) -> Result<()> {
@@ -16,6 +15,7 @@ pub fn exec(origin: &str) -> Result<()> {
     let conf = GitConfig {
         url: source.uri,
         branch: source.branch.clone(),
+        gates_branch: source.gates_branch.clone(),
         private_key: source.private_key,
         dir: origin.to_string(),
     };
@@ -25,16 +25,22 @@ pub fn exec(origin: &str) -> Result<()> {
             .environment
             .ok_or_else(|| anyhow!("Environment not specified in source"))
     })?;
-    let mut ws = Workspace::new(source.config)?;
+    let mut ws = Workspace::new(&config.scope, source.config)?;
     let env = config
         .environments
         .get(&environment)
         .context(format!("Environment '{}' not found in config", environment))?;
-    let (head, diff) = ws.record_env(env, true, true, Some(conf))?;
+    let gate = get_gate(
+        source.gates_file.as_ref(),
+        source.gates_branch.as_ref(),
+        &environment,
+        &Repo::open(None)?,
+    )?;
+    let (new_head, diff) = ws.record_env(env, gate, true, true, Some(conf))?;
     println!(
         "{}",
         serde_json::to_string(&ResourceData {
-            version: Version { head },
+            version: Version { trigger: new_head },
             metadata: diff
                 .into_iter()
                 .map(|diff| DiffElem {
