@@ -4,18 +4,21 @@ use std::path::Path;
 
 pub struct Workspace {
     path_to_config: String,
+    scope: String,
     db: Database,
 }
+
 impl Workspace {
-    pub fn new(path_to_config: String) -> Result<Self> {
+    pub fn new(scope: &str, path_to_config: String) -> Result<Self> {
         Ok(Self {
-            db: Database::open(&path_to_config)?,
+            db: Database::open(scope, &path_to_config)?,
+            scope: scope.to_string(),
             path_to_config,
         })
     }
 
-    pub fn ls(&self, env: &EnvironmentConfig) -> Result<Vec<String>> {
-        let repo = Repo::open()?;
+    pub fn ls(&self, env: &EnvironmentConfig, gate: Option<String>) -> Result<Vec<String>> {
+        let repo = Repo::open(gate)?;
         let new_env_state = self.construct_env_state(&repo, env, false)?;
         Ok(new_env_state
             .files
@@ -24,8 +27,12 @@ impl Workspace {
             .collect())
     }
 
-    pub fn check(&self, env: &EnvironmentConfig) -> Result<Option<(String, Vec<FileDiff>)>> {
-        let repo = Repo::open()?;
+    pub fn check(
+        &self,
+        env: &EnvironmentConfig,
+        gate: Option<String>,
+    ) -> Result<Option<(String, Vec<FileDiff>)>> {
+        let repo = Repo::open(gate)?;
         if let Some(previous_env) = env.propagated_from() {
             self.db.get_current_state(previous_env).context(format!(
                 "Previous environment '{}' not deployed yet",
@@ -60,19 +67,14 @@ impl Workspace {
                 eprintln!("File {} was removed", name)
             }
         }
-        Ok(Some((new_env_state.head_commit.to_short_ref(), diffs)))
+        Ok(Some((new_env_state.head_commit.inner(), diffs)))
     }
 
     pub fn reproduce(&self, env: &EnvironmentConfig, force_clean: bool) -> Result<()> {
-        let repo = Repo::open()?;
+        let repo = Repo::open(None)?;
         if let Some(last_state) = self.db.get_current_state(&env.name) {
             if force_clean {
-                let file_names: Vec<String> = last_state
-                    .files
-                    .iter()
-                    .map(|(ident, _)| ident.name())
-                    .collect();
-                repo.checkout_head(Some(&file_names), &self.ignore_list())?;
+                repo.checkout_gate(&[], &self.ignore_list(), true)?;
             }
             for (ident, state) in last_state.files.iter() {
                 repo.checkout_file_from(&ident.name(), &state.from_commit)?;
@@ -82,25 +84,25 @@ impl Workspace {
             Err(anyhow!("No state recorded for {}", env.name))
         }
     }
-    pub fn prepare(&self, env: &EnvironmentConfig, force_clean: bool) -> Result<()> {
-        let repo = Repo::open()?;
-        let head_files = if force_clean {
-            Some(env.head_filters())
-        } else {
-            None
-        };
+    pub fn prepare(
+        &self,
+        env: &EnvironmentConfig,
+        gate: Option<String>,
+        force_clean: bool,
+    ) -> Result<()> {
+        let repo = Repo::open(gate)?;
         let ignore_list = self.ignore_list();
-        repo.checkout_head(head_files, &ignore_list)?;
         let head_patterns: Vec<_> = env.head_file_patterns().collect();
+        repo.checkout_gate(&head_patterns, &ignore_list, force_clean)?;
         for file_buf in env.propagated_files() {
-            let file = file_buf.to_str().unwrap().to_string();
-            if !ignore_list
-                .iter()
-                .any(|p| p.matches_with(&file, Self::match_options()))
+            let file = file_buf.as_path();
+            if file.is_file()
+                && !ignore_list
+                    .iter()
+                    .any(|p| p.matches_path_with(file, MATCH_OPTIONS))
                 && !head_patterns
                     .iter()
-                    .any(|p| p.matches_with(&file, Self::match_options()))
-                && Path::new(&file).is_file()
+                    .any(|p| p.matches_path_with(file, MATCH_OPTIONS))
             {
                 std::fs::remove_file(file_buf).expect("Couldn't remove file");
             }
@@ -115,10 +117,10 @@ impl Workspace {
                     let name = ident.name();
                     if patterns
                         .iter()
-                        .any(|p| p.matches_with(&name, Self::match_options()))
+                        .any(|p| p.matches_with(&name, MATCH_OPTIONS))
                         && !head_patterns
                             .iter()
-                            .any(|p| p.matches_with(&name, Self::match_options()))
+                            .any(|p| p.matches_with(&name, MATCH_OPTIONS))
                     {
                         repo.checkout_file_from(&name, &state.from_commit)?;
                     }
@@ -131,14 +133,15 @@ impl Workspace {
     pub fn record_env(
         &mut self,
         env: &EnvironmentConfig,
+        gate: Option<String>,
         commit: bool,
         reset: bool,
         git_config: Option<GitConfig>,
     ) -> Result<(String, Vec<FileDiff>)> {
         eprintln!("Recording current state");
-        let repo = Repo::open()?;
+        let repo = Repo::open(gate)?;
         let new_env_state = self.construct_env_state(&repo, env, true)?;
-        let head_commit = new_env_state.head_commit.to_short_ref();
+        let head_commit = new_env_state.head_commit.clone().inner();
         let diffs = if let Some(last_state) = self.db.get_current_state(&env.name) {
             new_env_state.diff(last_state)
         } else {
@@ -163,7 +166,7 @@ impl Workspace {
         }
         if reset {
             eprintln!("Reseting head to have a clean workspace");
-            repo.checkout_head(None, &Vec::new())?;
+            repo.checkout_head()?;
         }
         if let Some(config) = git_config {
             eprintln!("Pushing to remote");
@@ -179,9 +182,10 @@ impl Workspace {
         env: &EnvironmentConfig,
         recording: bool,
     ) -> Result<DeployState> {
-        let current_commit = repo.head_commit_hash()?;
+        let current_commit = repo.gate_commit_hash();
         let database = Database::open_env(
             &self.path_to_config,
+            &self.scope,
             &env.name,
             env.propagated_from(),
             current_commit.clone(),
@@ -231,6 +235,7 @@ impl Workspace {
         };
         let database = Database::open_env(
             &self.path_to_config,
+            &config.scope,
             &env.name,
             env.propagated_from(),
             commit.clone(),
@@ -264,7 +269,7 @@ impl Workspace {
                     if let Some(last_hash) = prev_state.file_hash.as_ref() {
                         if patterns
                             .iter()
-                            .any(|p| p.matches_with(&name, Self::match_options()))
+                            .any(|p| p.matches_with(&name, MATCH_OPTIONS))
                         {
                             let (dirty, file_hash) = if recording {
                                 if let Some(file_hash) = hash_file(&name) {
@@ -297,10 +302,10 @@ impl Workspace {
         repo.all_files(commit.clone(), |file_hash, path| {
             if env
                 .head_file_patterns()
-                .any(|p| p.matches_path_with(path, Self::match_options()))
+                .any(|p| p.matches_path_with(path, MATCH_OPTIONS))
                 && !ignore_list
                     .iter()
-                    .any(|p| p.matches_path_with(path, Self::match_options()))
+                    .any(|p| p.matches_path_with(path, MATCH_OPTIONS))
             {
                 let (from_commit, message) = repo.find_last_changed_commit(path, commit.clone())?;
                 let state = if recording {
@@ -345,12 +350,4 @@ impl Workspace {
             glob::Pattern::new(".gitignore").unwrap(),
         ]
     }
-
-    fn match_options() -> glob::MatchOptions {
-        glob::MatchOptions {
-            case_sensitive: true,
-            require_literal_separator: true,
-            require_literal_leading_dot: true,
-        }
-    }
 }
